#ifndef REMOTEALGMANAGER_H
#define REMOTEALGMANAGER_H

#include <afxwin.h>         
#include "..\Shared\status.h"
#include "WAESCipher_Auth.h"
#include "WAESCipher_Trans.h"
#include "SecureFWGlobals.h"
#include "APDUManager.h"

// INICIALIZATION VECTOR SHARED BETWEEN SWA AND HWT
const unsigned char IV[16] = {0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c};

class CRemoteAlgManager {
private:
    CAPDUManager        m_apduManager;                  // CLASS FOR APDU BASED COMMUNICATION WITH HW TOKEN
    CWAESCipher_Auth    m_aesCipherAuth;                // WBACR AES CIPHER (ENCRYPT & DECRYPT PART) USED DURING AUTHENTIZATION PART OF SEAUT
    CWAESCipher_Trans   m_aesCipherTrans;               // WBACR AES CIPHER (ENCRYPT & DECRYPT PART) USED DURING TRANSPORT PART OF SEAUT
    BYTE                m_sessionKey[SESSION_KEY_LEN];  // SESSION KEY VALUE USED DURING TRANSPORT PART OF SEAUT

public:
    CRemoteAlgManager();
    ~CRemoteAlgManager();

    // STEPS TO USE PROTECTED ALGORITHM:
    // 1. Connect to hardware token ::Connect()
    // 2. Authenticate software agent ::ProcessAuth()
    // 3. Send & receive algorithm data ::ProcessAlgData()
    // 4. Finalize authenticated communication ::StopAuth()
    // 5. Disconnect from hardware token ::Disconnect()

    // CONNECT TO TARGET HARDWARE TOKEN
    int Connect(REMOTE_APPLET* pRemoteApplet);                      
    // DISCONNECT FROM ACTUAL HARDWARE TOKEN
    int Disconnect();                                               
    
    // PROCESS AUTHENTIZATION AS DEFINED BY SEAUT
    int ProcessAuth(SWA_PROFILE* pSWAProfile, BYTE debugFlags = 0); 
    // FINALIZE PREVIOUSLY PERFORMED AUTHENTIZATION
    int StopAuth(BYTE debugFlags = 0);                              
                                                                    
    // SEND PROTECTED ALGORITHM INPUT DATA & RECEIVE RESPONSE, PREVIOUS AUTHENTIZATION IS REQUIRED
    int ProcessAlgData(SHORT_DATA_BLOB* pAlgUUID, SHORT_DATA_BLOB* pInputData, SHORT_DATA_BLOB* pOutputData, BYTE debugFlags = 0);
         
                                                                    
    // STEPS TO SEND XML LICENCE
    // 1. Connect to hardware token ::Connect()
    // 2. Send licence ::ProcessMasterLicence() or ::ProcessAlgLicence()
    // 3. Disconnect from hardware token ::Disconnect()
    
    // SEND MASTER LICENCE, SET outOfOrder FLAG, IF LICENCE ISN'T IN ORDINARY ORDER 
    int ProcessMasterLicence(MASTER_LICENCE* pMasterLicence, BOOL outOfOrder = FALSE, BYTE debugFlags = 0); 
    // SEND ALGORITHM LICENCE, SET outOfOrder FLAG, IF LICENCE ISN'T IN ORDINARY ORDER 
    int ProcessAlgLicence(ALG_LICENCE* pAlgLicence, BOOL outOfOrder = FALSE, BYTE debugFlags = 0); 
    

    // RESET ALL INTERNAL APPLET VALUES, AVAILABLE ONLY IN DEBUG MODE OF HARDWARE TOKEN
    int DebugReset();
protected:
    // UPDATE SESSION KEY VALUE (AS DEFINED IN SEAUT)
    int UpdateSessionKey(BYTE* pSessionKey);

    // SECURE OUTGOING REQUEST TO PROTECTED ALGORITHM (AS DEFINED IN SEAUT)
    int SecureRequest(BYTE* pRequest, DWORD* pbRequestLen, DWORD* pbRequestLenMax);
    // UNSECURE INCOMING RESPONSE FROM PROTECTED ALGORITHM (AS DEFINED IN SEAUT)
    int UnsecureResponse(BYTE* pResponse, DWORD* pbResponseLen);

private:
    SWA_PROFILE     m_actualSWAProfile;
    REMOTE_APPLET   m_actualRemoteApplet;
};

#endif